* Native colour conversion
*
* V0.01, pjwitte 2oi8+
* V0.02, pjw, Nov 21 2018, Nybble-based Aurora RGB table
*
* Compatible with all GD2 modes
*


        section code

        xdef ut_cv16to24
        xdef cv_16to24

*
ut_cv16to24
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* entry:
*       d3.b = grbgrbgx                 - Aur (mode 16) colour code
*
* exit:
*       d3.l = 0 R G B                  - 24 bit colour <> mode 64!
*
*       d0/d5/d6/a1/a3 smashed
*
*       no errors; always "suceeds"

        lea.l tab_aur8rgbn,a3

*
cv_16to24
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* This version uses a smaller RGB table, taking advantage of the fact that
* only the MS nybble of each colour element is significant. Thus the table
* consists of the Aurora RGB palette in 3 x 3 nybble format - a 50% space
* saving, although computationally more demanding.
*
* entry:
*       d3.b = grbgrbgx        - Aur (mode 16) colour code
*
* exit:
*       d3.l = 0 R G B         - 24 bit colour <> mode 64!
*
*       d0/d5/d6/a1 smashed
*
* No errors; always "suceeds"

        move.l #$ff,d6          clear out (most of) d6
        and.w d6,d3             byte -> word
        move.w d3,d5            00 CC
        add.w d3,d5             cc cc
        add.w d3,d5             CC CC
        lsr.w #1,d5             0C CC

        lea.l (a3,d5.w),a1      od   ev
        move.b (a1)+,d6         xR / RG
        move.b (a1),d0          GB / Bx

        btst #0,d3
        beq.s even

odd
        lsl.b #4,d6             R0
        move.b d0,d3            GB
        andi.b #$f0,d3          G0
        andi.b #$0f,d0          0B
        lsl.b #4,d0             B0
        bra.s finalise

even
        move.b d6,d3            rG
        lsl.b #4,d3             G0
        andi.b #$f0,d6          R0
        andi.b #$f0,d0          B0

finalise
        swap d6                 00R0 | xxxx
        move.b d3,d6
        lsl.w #8,d6             00R0 | G0xx
        move.b d0,d6            00R0 | G0B0

done
        move.l d6,d3            fudge
        rts


*
* Aurora RGB palette in a 3 x 3 nybble format
*
tab_aur8rgbn
        dc.b $00,$02,$00,$02,$02,$20,$00,$40,$06,$02,$40,$26
        dc.b $40,$06,$00,$42,$06,$20,$40,$46,$06,$42,$46,$26
        dc.b $04,$02,$40,$06,$02,$60,$04,$40,$46,$06,$40,$66
        dc.b $44,$06,$40,$46,$06,$60,$44,$46,$46,$46,$46,$66
        dc.b $00,$80,$0A,$02,$80,$2A,$00,$C0,$0E,$02,$C0,$2E
        dc.b $40,$86,$0A,$42,$86,$2A,$40,$C6,$0E,$42,$C6,$2E
        dc.b $04,$80,$4A,$06,$80,$6A,$04,$C0,$4E,$06,$C0,$6E
        dc.b $44,$86,$4A,$46,$86,$6A,$44,$C6,$4E,$46,$C6,$6E
        dc.b $80,$0A,$00,$82,$0A,$20,$80,$4A,$06,$82,$4A,$26
        dc.b $C0,$0E,$00,$C2,$0E,$20,$C0,$4E,$06,$C2,$4E,$26
        dc.b $84,$0A,$40,$86,$0A,$60,$84,$4A,$46,$86,$4A,$66
        dc.b $C4,$0E,$40,$C6,$0E,$60,$C4,$4E,$46,$C6,$4E,$66
        dc.b $80,$8A,$0A,$82,$8A,$2A,$80,$CA,$0E,$82,$CA,$2E
        dc.b $C0,$8E,$0A,$C2,$8E,$2A,$C0,$CE,$0E,$C2,$CE,$2E
        dc.b $84,$8A,$4A,$86,$8A,$6A,$84,$CA,$4E,$86,$CA,$6E
        dc.b $C4,$8E,$4A,$C6,$8E,$6A,$C4,$CE,$4E,$C6,$CE,$6E
        dc.b $08,$02,$80,$0A,$02,$A0,$08,$40,$86,$0A,$40,$A6
        dc.b $48,$06,$80,$4A,$06,$A0,$48,$46,$86,$4A,$46,$A6
        dc.b $0C,$02,$C0,$0E,$02,$E0,$0C,$40,$C6,$0E,$40,$E6
        dc.b $4C,$06,$C0,$4E,$06,$E0,$4C,$46,$C6,$4E,$46,$E6
        dc.b $08,$80,$8A,$0A,$80,$AA,$08,$C0,$8E,$0A,$C0,$AE
        dc.b $48,$86,$8A,$4A,$86,$AA,$48,$C6,$8E,$4A,$C6,$AE
        dc.b $0C,$80,$CA,$0E,$80,$EA,$0C,$C0,$CE,$0E,$C0,$EE
        dc.b $4C,$86,$CA,$4E,$86,$EA,$4C,$C6,$CE,$4E,$C6,$EE
        dc.b $88,$0A,$80,$8A,$0A,$A0,$88,$4A,$86,$8A,$4A,$A6
        dc.b $C8,$0E,$80,$CA,$0E,$A0,$C8,$4E,$86,$CA,$4E,$A6
        dc.b $8C,$0A,$C0,$8E,$0A,$E0,$8C,$4A,$C6,$8E,$4A,$E6
        dc.b $CC,$0E,$C0,$CE,$0E,$E0,$CC,$4E,$C6,$CE,$4E,$E6
        dc.b $88,$8A,$8A,$8A,$8A,$AA,$88,$CA,$8E,$8A,$CA,$AE
        dc.b $C8,$8E,$8A,$CA,$8E,$AA,$C8,$CE,$8E,$CA,$CE,$AE
        dc.b $8C,$8A,$CA,$8E,$8A,$EA,$8C,$CA,$CE,$8E,$CA,$EE
        dc.b $CC,$8E,$CA,$CE,$8E,$EA,$CC,$CE,$CE,$CE,$CE,$EE

*

        end
