* Subroutines for Quicksort and Binary search
*
* pjwitte 2o23+
* All code is my own work except as and where stated
*
* V0.01, pjw, 2023 Jul 27
*
* This version works with the RI stack pointer (a6)
* Compatible with QDOS, Minerva and SMSQ/E
*


        section code

        xdef cmpel

        include dev8_keys_qlv


* Types:
*
*   sbyte = $00                  - signed byte
*   ubyte = $02                  - unsigned byte
*   sword = $04                  - signed 16 bit word
*   uword = $06                  - unsigned 16 bit word
*   slong = $08                  - signed 32 bit long word
*   ulong = $0A                  - unsigned 32 bit long word
*   fltpt = $0C                  - 48 bit float
*   qstrg = $0E                  - Q-string (len.w + bytes)
*
* Variable types will have been checked for validity and range before
* entry

*
cmpel
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* Compare elements by type:
*
*  d0.l =               result -1 (X < Y), 0 (X = Y), 1 (X > Y)
*  d1                   smashed
*  d5.l = type          saved    0.comp type.b | 0.variable type.b
*  a0  -> X             saved
*  a1  -> Y             saved
*  a2                   smashed
*  a6                   saved    base register
*
*

        move.w tab(pc,d5.w),d0
        jmp tab(pc,d0.w)            jump to it

*
tab
        dc.w sbyte_cmp-tab          $00
        dc.w ubyte_cmp-tab          $02
        dc.w sword_cmp-tab          $04
        dc.w uword_cmp-tab          $06
        dc.w slong_cmp-tab          $08
        dc.w ulong_cmp-tab          $0A
        dc.w fltpt_cmp-tab          $0C
        dc.w qstrg_cmp-tab          $0E

*
sbyte_cmp
        move.b 1(a6,a1.l),d0
        cmp.b 1(a6,a0.l),d0
        bra.s signed

*
sword_cmp
        move.w (a6,a1.l),d0
        cmp.w (a6,a0.l),d0
        bra.s signed

*
slong_cmp
        move.l (a6,a1.l),d0
        cmp.l (a6,a0.l),d0

signed
        beq.s eq
        blt.s mi
        bgt.s pl

*
ubyte_cmp
        move.b 1(a6,a1.l),d0
        cmp.b 1(a6,a0.l),d0
        bra.s unsigned

*
uword_cmp
        move.w (a6,a1.l),d0
        cmp.w (a6,a0.l),d0
        bra.s unsigned

*
ulong_cmp
        move.l (a6,a1.l),d0
        cmp.l (a6,a0.l),d0

unsigned
        beq.s eq
        bcs.s mi
        bhi.s pl

*
qstrg_cmp
         move.l d5,d0
         swap d0                        Qdos comparison type
         move.w ut.cstr,a2              jsr ut.cstr*3+qlv.ofs+2
         jmp (a2)

*
fltpt_cmp                               ; cmp.f f1,f0; where
        move.l 2(a6,a1.l),d0
        cmp.l 2(a6,a0.l),d0
        bvc.s flt_ss                    sign same

        tst.l d0                        signs different so
        bra.s flt_done                   return comparison

flt_ss
        tst.l d0                        signs positive?
        bmi.s flt_neg

        move.w (a6,a1.l),d1
        cmp.w (a6,a0.l),d1                  how do exps compare?
        bne.s flt_done                   different magnitude

        cmp.l 2(a6,a0.l),d0                   same magnitude

flt_done
        bmi.s mi
        beq.s eq

pl
        moveq #1,d0
        rts

eq
        moveq #0,d0
        rts

mi
        moveq #-1,d0
        rts

flt_neg
        move.w (a6,a0.l),d1
        cmp.w (a6,a1.l),d1
        bne.s flt_done

        cmp.l 2(a6,a0.l),d0
        bra.s flt_done

*
        end
